@@ -1,4 +1,5 @@
 from abc import ABC, abstractmethod, abstractproperty
+import sys
 import git, gitdb
 from os.path import basename
 from bossman.errors import BossmanError
@@ -14,7 +15,11 @@ def true(*args, **kwargs):
 def short_rev(rev):
   return rev[:8] if rev else rev
 
-class RepoError(RuntimeError):
+class RepoError(BossmanError):
+  pass
+class RepoAuthError(RepoError):
+  pass
+class RepoRevNotFoundError(RepoError):
   pass
 
 class TreeVisitor:
@@ -298,7 +303,21 @@ class Repo:
     for section in conf.sections():
       if section.startswith("remote"):
         remote = section.split(" ").pop().strip('"')
-        self._repo.git.fetch(remote, "+{}:{}".format(ref, ref))
+        try:
+          self._repo.git.fetch(remote, "+{}:{}".format(ref, ref))
+        except git.GitCommandError as e:
+          if "invalid refspec" in e.stderr:
+            # if we don't have notes in this namespace we might get
+            # an invalid refspec
+            print("Warning: invalid refspec error occurred when running `{}`".format(e._cmdline), file=sys.stderr)
+            pass
+          elif "couldn't find remote ref" in e.stderr:
+            # if the remote doesn't know about this ref yet, this can happen; it is fine
+            pass
+          elif "authentication failures" in e.stderr:
+            raise RepoAuthError("Failed to fetch from remote {}: {}".format(remote, e.stderr))
+          else:
+            raise e
 
   @synchronized
   def push_notes(self, ns: str = "*"):
@@ -307,7 +326,21 @@ class Repo:
     for section in conf.sections():
       if section.startswith("remote"):
         remote = section.split(" ").pop().strip('"')
-        self._repo.git.push(remote, "+{}:{}".format(ref, ref))
+        try:
+          self._repo.git.push(remote, "+{}:{}".format(ref, ref))
+        except git.GitCommandError as e:
+          if "invalid refspec" in e.stderr:
+            # if we don't have notes in this namespace we might get
+            # an invalid refspec; this should be quite unlikely
+            print("Warning: invalid refspec error occurred when running `{}`".format(e._cmdline), file=sys.stderr)
+            pass
+          elif "does not match any" in e.stderr:
+            # if we did not write any notes for this ns, we will get this error
+            pass
+          elif "authentication failures" in e.stderr:
+            raise RepoAuthError("Failed to fetch from remote {}: {}".format(remote, e.stderr))
+          else:
+            raise e
 
   @synchronized
   def rev_parse(self, rev: str) -> str:
@@ -317,7 +350,7 @@ class Repo:
       hexsha = self._repo.git.rev_parse(rev)
       return self._repo.rev_parse(hexsha)
     except (git.GitCommandError, gitdb.exc.BadName):
-      raise BossmanError("failed to resolve revision {}, please make sure it is a valid commit/tag name".format(rev))
+      raise RepoRevNotFoundError("failed to resolve revision {}, please make sure it is a valid commit/tag name".format(rev))
 
   @synchronized
   def rev_exists(self, rev: str) -> str:
@@ -347,14 +380,12 @@ class Repo:
     Lists the paths versioned for revision {rev}, optionally filtered
     by {predicate}.
     """
-    try:
-      commit = self.rev_parse(rev)
-    except gitdb.exc.BadName:
-      raise BossmanError("This branch likely has no commits yet.")
+    commit = self.rev_parse(rev)
+    tree  = commit.tree
 
     paths = []
     visitor = TreeVisitor(lambda blob: paths.append(blob.path))
-    visitor(commit.tree)
+    visitor(tree)
     return paths
 
   @synchronized
